﻿USE tempdb;
GO

-- zsumuj sprzedaż na produkt, rok, miesiąc, dzień
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	Rok			= YEAR(s.data_sprzedazy),
	Miesiac		= MONTH(s.data_sprzedazy),
	Dzien		= DAY(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Produkt,
	Rok,
	Miesiac,
	Dzien,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY
	Produkt,
	Rok,
	Miesiac,
	Dzien
ORDER BY
	Produkt,
	Rok,
	Miesiac,
	Dzien

-- no rewelacja, a gdzie GROUPING SET?

-- każdy zestaw kolumn w GROUP BY traktujemy jako zbiór grupujący
-- czyli możemy napisać:

;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	Rok			= YEAR(s.data_sprzedazy),
	Miesiac		= MONTH(s.data_sprzedazy),
	Dzien		= DAY(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Produkt,
	Rok,
	Miesiac,
	Dzien,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY GROUPING SETS (
	(Produkt, Rok, Miesiac, Dzien)
)
ORDER BY
	Produkt,
	Rok,
	Miesiac,
	Dzien

-- nadal raczej nic szalonego, to może zgrupujmy dane na kilku poziomach? Np. dodaj sumę w danym roku dla produktu?
-- robimy jak znamy:
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	Rok			= YEAR(s.data_sprzedazy),
	Miesiac		= MONTH(s.data_sprzedazy),
	Dzien		= DAY(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Produkt,
	Rok,
	Miesiac,
	Dzien,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY Produkt, Rok, Miesiac, Dzien

UNION ALL

SELECT
	Produkt,
	Rok,
	Miesiac		= NULL,
	Dzien		= NULL,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY Produkt, Rok


ORDER BY
	Produkt,
	Rok,
	Miesiac,
	Dzien

-- a jak dodamy grupowanie po miesiącach?
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	Rok			= YEAR(s.data_sprzedazy),
	Miesiac		= MONTH(s.data_sprzedazy),
	Dzien		= DAY(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Produkt,
	Rok,
	Miesiac,
	Dzien,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY Produkt, Rok, Miesiac, Dzien

UNION ALL

SELECT
	Produkt,
	Rok,
	Miesiac		= NULL,
	Dzien		= NULL,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY Produkt, Rok

UNION ALL

SELECT
	Produkt,
	Rok,
	Miesiac,
	Dzien		= NULL,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY Produkt, Rok, Miesiac


ORDER BY
	Produkt,
	Rok,
	Miesiac,
	Dzien

-- GROUPING SET to każde takie grupowanie danych wymienione w UNION ALL
-- i zamiast pisać tyle kodu możemy to skrócić:
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	Rok			= YEAR(s.data_sprzedazy),
	Miesiac		= MONTH(s.data_sprzedazy),
	Dzien		= DAY(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Produkt,
	Rok,
	Miesiac,
	Dzien,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY GROUPING SETS (
	(Produkt, Rok, Miesiac, Dzien),
	(Produkt, Rok),
	(Produkt, Rok, Miesiac),
	()						-- <-- łatwo też da się wyliczyć TOTAL
)
ORDER BY
	Produkt,
	Rok,
	Miesiac,
	Dzien

-- czyli jeśli mam dużo kolumn i chcę zrobić wszystkie kombinacje
-- to trochę pisania będzie?

-- to zależy jak rozumiemy wszystkie kombinacje:

-- 1) wszystkie wszystkie (2^N)
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	Rok			= YEAR(s.data_sprzedazy),
	Miesiac		= MONTH(s.data_sprzedazy),
	Dzien		= DAY(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Produkt,
	Rok,
	Miesiac,
	Dzien,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY CUBE (Produkt, Rok, Miesiac, Dzien)
ORDER BY
	Produkt,
	Rok,
	Miesiac,
	Dzien

-- 1a) wycofywana składnia
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	Rok			= YEAR(s.data_sprzedazy),
	Miesiac		= MONTH(s.data_sprzedazy),
	Dzien		= DAY(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Produkt,
	Rok,
	Miesiac,
	Dzien,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY Produkt, Rok, Miesiac, Dzien
	WITH CUBE
ORDER BY
	Produkt,
	Rok,
	Miesiac,
	Dzien

-- 2) W kolumnach mam hierarchię (np. Rok->Miesiąc->Dzień)
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	Rok			= YEAR(s.data_sprzedazy),
	Miesiac		= MONTH(s.data_sprzedazy),
	Dzien		= DAY(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Produkt,
	Rok,
	Miesiac,
	Dzien,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY ROLLUP (Produkt, Rok, Miesiac, Dzien)
ORDER BY
	Produkt,
	Rok,
	Miesiac,
	Dzien

-- 2a) wycofywana składnia
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	Rok			= YEAR(s.data_sprzedazy),
	Miesiac		= MONTH(s.data_sprzedazy),
	Dzien		= DAY(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Produkt,
	Rok,
	Miesiac,
	Dzien,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY Produkt, Rok, Miesiac, Dzien
	WITH ROLLUP
ORDER BY
	Produkt,
	Rok,
	Miesiac,
	Dzien





-- a czy zawsze musi być ten podstawowy poziom grupowania po każdej kolumnie?
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	Rok			= YEAR(s.data_sprzedazy),
	Miesiac		= MONTH(s.data_sprzedazy),
	Dzien		= DAY(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Produkt,
	Rok,
	Miesiac,
	Dzien,
	sumaIlosc	= SUM(ilosc),
	sumaWartosc	= SUM(wartosc)
FROM Dane d
GROUP BY GROUPING SETS (
--	(Produkt, Rok, Miesiac, Dzien),
	(Produkt, Dzien),
	(Produkt, Rok),
	(Produkt, Rok, Miesiac),
	()
)
ORDER BY
	Produkt,
	Rok,
	Miesiac,
	Dzien
