﻿USE tempdb;
GO

-- Ten PIVOT jest nieco ograniczony - można zrobić tylko jeden agregat, 
-- nie mogę zrobić np. jednoczesnego sumowania po ilości i wartości:
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	RokMiesiac	= YEAR(s.data_sprzedazy) * 100 + MONTH(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)

SELECT
	Pracownik,
	Produkt,
	[201401], [201402], [201403], [201404], [201405], [201406],
	[201407], [201408], [201409], [201410], [201411], [201412]
FROM Dane

	PIVOT (
		SUM(ilosc)
		FOR RokMiesiac IN (
			[201401], [201402], [201403], [201404], [201405], [201406],
			[201407], [201408], [201409], [201410], [201411], [201412]
		)
	) p

	PIVOT (
		SUM(wartosc)
		FOR RokMiesiac IN (
			[201401], [201402], [201403], [201404], [201405], [201406],
			[201407], [201408], [201409], [201410], [201411], [201412]
		)
	) p1

-- jeśli chcemy napisać kod, który np. zsumuje od razu ilości i wartości, 
-- to musimy napisać masę kodu:
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	RokMiesiac	= YEAR(s.data_sprzedazy) * 100 + MONTH(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Pracownik,
	Produkt,
	[201401_ilosc]		= SUM(CASE WHEN RokMiesiac = 201401 THEN ilosc END),
	[201402_ilosc]		= SUM(CASE WHEN RokMiesiac = 201402 THEN ilosc END),
	[201403_ilosc]		= SUM(CASE WHEN RokMiesiac = 201403 THEN ilosc END),
	[201404_ilosc]		= SUM(CASE WHEN RokMiesiac = 201404 THEN ilosc END),
	[201405_ilosc]		= SUM(CASE WHEN RokMiesiac = 201405 THEN ilosc END),
	[201406_ilosc]		= SUM(CASE WHEN RokMiesiac = 201406 THEN ilosc END),
	[201407_ilosc]		= SUM(CASE WHEN RokMiesiac = 201407 THEN ilosc END),
	[201408_ilosc]		= SUM(CASE WHEN RokMiesiac = 201408 THEN ilosc END),
	[201409_ilosc]		= SUM(CASE WHEN RokMiesiac = 201409 THEN ilosc END),
	[201410_ilosc]		= SUM(CASE WHEN RokMiesiac = 201410 THEN ilosc END),
	[201411_ilosc]		= SUM(CASE WHEN RokMiesiac = 201411 THEN ilosc END),
	[201412_ilosc]		= SUM(CASE WHEN RokMiesiac = 201412 THEN ilosc END),

	[201401_wartosc]	= SUM(CASE WHEN RokMiesiac = 201401 THEN wartosc END),
	[201402_wartosc]	= SUM(CASE WHEN RokMiesiac = 201402 THEN wartosc END),
	[201403_wartosc]	= SUM(CASE WHEN RokMiesiac = 201403 THEN wartosc END),
	[201404_wartosc]	= SUM(CASE WHEN RokMiesiac = 201404 THEN wartosc END),
	[201405_wartosc]	= SUM(CASE WHEN RokMiesiac = 201405 THEN wartosc END),
	[201406_wartosc]	= SUM(CASE WHEN RokMiesiac = 201406 THEN wartosc END),
	[201407_wartosc]	= SUM(CASE WHEN RokMiesiac = 201407 THEN wartosc END),
	[201408_wartosc]	= SUM(CASE WHEN RokMiesiac = 201408 THEN wartosc END),
	[201409_wartosc]	= SUM(CASE WHEN RokMiesiac = 201409 THEN wartosc END),
	[201410_wartosc]	= SUM(CASE WHEN RokMiesiac = 201410 THEN wartosc END),
	[201411_wartosc]	= SUM(CASE WHEN RokMiesiac = 201411 THEN wartosc END),
	[201412_wartosc]	= SUM(CASE WHEN RokMiesiac = 201412 THEN wartosc END)
FROM Dane
GROUP BY
	Pracownik,
	Produkt


-- można go nieco skrócić stosując macierz pomocniczą
-- (podpatrzone u Itzika Ben-Gana)
IF OBJECT_ID(N'dbo.PivotMatrix', N'U') IS NOT NULL
	EXEC sp_executesql N'DROP TABLE dbo.PivotMatrix';
GO

CREATE TABLE dbo.PivotMatrix (
	RokMiesiac	INT,
	rm201401	INT,
	rm201402	INT,
	rm201403	INT,
	rm201404	INT,
	rm201405	INT,
	rm201406	INT,
	rm201407	INT,
	rm201408	INT,
	rm201409	INT,
	rm201410	INT,
	rm201411	INT,
	rm201412	INT
)

INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201401) VALUES (201401, 1);
INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201402) VALUES (201402, 1);
INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201403) VALUES (201403, 1);
INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201404) VALUES (201404, 1);
INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201405) VALUES (201405, 1);
INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201406) VALUES (201406, 1);
INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201407) VALUES (201407, 1);
INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201408) VALUES (201408, 1);
INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201409) VALUES (201409, 1);
INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201410) VALUES (201410, 1);
INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201411) VALUES (201411, 1);
INSERT INTO dbo.PivotMatrix (RokMiesiac, rm201412) VALUES (201412, 1);

SELECT * FROM dbo.PivotMatrix

-- (w sumie też musieliśmy się napisać, ale tylko raz)
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	RokMiesiac	= YEAR(s.data_sprzedazy) * 100 + MONTH(s.data_sprzedazy),
	s.ilosc,
	s.wartosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)
SELECT
	Pracownik,
	Produkt,
	[201401_ilosc]		= SUM(d.ilosc * m.rm201401),
	[201402_ilosc]		= SUM(d.ilosc * m.rm201402),
	[201403_ilosc]		= SUM(d.ilosc * m.rm201403),
	[201404_ilosc]		= SUM(d.ilosc * m.rm201404),
	[201405_ilosc]		= SUM(d.ilosc * m.rm201405),
	[201406_ilosc]		= SUM(d.ilosc * m.rm201406),
	[201407_ilosc]		= SUM(d.ilosc * m.rm201407),
	[201408_ilosc]		= SUM(d.ilosc * m.rm201408),
	[201409_ilosc]		= SUM(d.ilosc * m.rm201409),
	[201410_ilosc]		= SUM(d.ilosc * m.rm201410),
	[201411_ilosc]		= SUM(d.ilosc * m.rm201411),
	[201412_ilosc]		= SUM(d.ilosc * m.rm201412),

	[201401_wartosc]	= SUM(d.wartosc * m.rm201401),
	[201402_wartosc]	= SUM(d.wartosc * m.rm201402),
	[201403_wartosc]	= SUM(d.wartosc * m.rm201403),
	[201404_wartosc]	= SUM(d.wartosc * m.rm201404),
	[201405_wartosc]	= SUM(d.wartosc * m.rm201405),
	[201406_wartosc]	= SUM(d.wartosc * m.rm201406),
	[201407_wartosc]	= SUM(d.wartosc * m.rm201407),
	[201408_wartosc]	= SUM(d.wartosc * m.rm201408),
	[201409_wartosc]	= SUM(d.wartosc * m.rm201409),
	[201410_wartosc]	= SUM(d.wartosc * m.rm201410),
	[201411_wartosc]	= SUM(d.wartosc * m.rm201411),
	[201412_wartosc]	= SUM(d.wartosc * m.rm201412)
FROM Dane d

	INNER JOIN dbo.PivotMatrix m
		ON m.RokMiesiac = d.RokMiesiac

GROUP BY
	Pracownik,
	Produkt


-- No dobra, super i fajnie - ale czemu muszę podawać daty?
-- Nie da się tego zrobić jakoś dynamicznie?
-- (sposób też podpatrzony u Itzika, ale jest też sporo innych podejść)
DECLARE
  @cols AS NVARCHAR(1000),
  @sql  AS NVARCHAR(4000);

SET @cols =
  STUFF(
    (SELECT N',' + QUOTENAME(RokMiesiac) AS [text()]
     FROM (SELECT DISTINCT RokMiesiac = YEAR(s.data_sprzedazy) * 100 + MONTH(s.data_sprzedazy)
           FROM dbo.Sprzedaz s) Okresy
     ORDER BY RokMiesiac
     FOR XML PATH(''), TYPE).value('.[1]', 'VARCHAR(MAX)'), 1, 1, '')

SET @sql = N'
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	RokMiesiac	= YEAR(s.data_sprzedazy) * 100 + MONTH(s.data_sprzedazy),
	s.ilosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)

SELECT
	Pracownik,
	Produkt,
	' + @cols + N'
FROM Dane

	PIVOT (
		SUM(ilosc)
		FOR RokMiesiac IN (' + @cols + N')
	) p';
--PRINT @sql;

EXEC sys.sp_executesql @stmt = @sql;
GO


-- na koniec utrwal wynik, bo będziemy wykorzystywać do zrobienia UNPIVOT
IF OBJECT_ID(N'dbo.SprzedazPivot', N'U') IS NOT NULL
	EXEC sp_executesql N'DROP TABLE dbo.SprzedazPivot';
GO

DECLARE
  @cols AS NVARCHAR(1000),
  @sql  AS NVARCHAR(4000);

SET @cols =
  STUFF(
    (SELECT N',' + QUOTENAME(RokMiesiac) AS [text()]
     FROM (SELECT DISTINCT RokMiesiac = YEAR(s.data_sprzedazy) * 100 + MONTH(s.data_sprzedazy)
           FROM dbo.Sprzedaz s) Okresy
     ORDER BY RokMiesiac
     FOR XML PATH(''), TYPE).value('.[1]', 'VARCHAR(MAX)'), 1, 1, '')

SET @sql = N'
;WITH Dane AS (
SELECT
	Pracownik	= p.nazwa,
	Produkt		= pr.nazwa,
	RokMiesiac	= YEAR(s.data_sprzedazy) * 100 + MONTH(s.data_sprzedazy),
	s.ilosc
FROM Sprzedaz s

	INNER JOIN Pracownik p
		ON p.id = s.pracownik_id

	INNER JOIN Produkt pr
		ON pr.id = s.produkt_id
)

SELECT
	Pracownik,
	Produkt,
	' + @cols + N'
INTO dbo.SprzedazPivot
FROM Dane

	PIVOT (
		SUM(ilosc)
		FOR RokMiesiac IN (' + @cols + N')
	) p';
--PRINT @sql;

EXEC sys.sp_executesql @stmt = @sql;
GO